
#include "py/nlr.h"
#include "py/runtime.h"
#include "py/mphal.h"
#include "timer.h"
#include "pin.h"
#include "genhdr/pins.h"
#include "cq.h"

typedef struct _pyb_cq_obj_t {
    mp_obj_base_t base;
    mp_uint_t cq_id;
} pyb_cq_obj_t;

STATIC const pyb_cq_obj_t pyb_cq_obj[] = {
    {{&pyb_cq_type}, 1},
    {{&pyb_cq_type}, 2},
    {{&pyb_cq_type}, 3},
    {{&pyb_cq_type}, 4},
};

void cq_obj_print(const mp_print_t *print, mp_obj_t self_in, mp_print_kind_t kind) {
    pyb_cq_obj_t *self = self_in;
    mp_printf(print, "CQ(%lu)", self->cq_id);
}

STATIC mp_obj_t cq_obj_make_new(const mp_obj_type_t *type, mp_uint_t n_args, mp_uint_t n_kw, const mp_obj_t *args) {
    // check arguments
    mp_arg_check_num(n_args, n_kw, 1, 1, false);

    // get CQ number
    mp_int_t cq_id = mp_obj_get_int(args[0]);

    // check CQ number
    if (!(1 <= cq_id && cq_id <= 4)) {
        nlr_raise(mp_obj_new_exception_msg_varg(&mp_type_ValueError, "CQ(%d) does not exist", cq_id));
    }

    // return static CQ object
    return (mp_obj_t)&pyb_cq_obj[cq_id - 1];
}

void cq_state(mp_uint_t cq_id, int state) {
}

mp_obj_t cq_obj_on(mp_obj_t self_in) {
    pyb_cq_obj_t *self = self_in;
    cq_state(self->cq_id, 1);
    return mp_const_none;
}

mp_obj_t cq_obj_off(mp_obj_t self_in) {
    pyb_cq_obj_t *self = self_in;
    cq_state(self->cq_id, 0);
    return mp_const_none;
}
STATIC MP_DEFINE_CONST_FUN_OBJ_1(cq_obj_on_obj, cq_obj_on);
STATIC MP_DEFINE_CONST_FUN_OBJ_1(cq_obj_off_obj, cq_obj_off);

STATIC const mp_map_elem_t cq_locals_dict_table[] = {
    { MP_OBJ_NEW_QSTR(MP_QSTR_on), (mp_obj_t)&cq_obj_on_obj },
    { MP_OBJ_NEW_QSTR(MP_QSTR_off), (mp_obj_t)&cq_obj_off_obj },
};

STATIC MP_DEFINE_CONST_DICT(cq_locals_dict, cq_locals_dict_table);

const mp_obj_type_t pyb_cq_type = {
    { &mp_type_type },
    .name = MP_QSTR_CQ,
    .print = cq_obj_print,
    .make_new = cq_obj_make_new,
    .locals_dict = (mp_obj_t)&cq_locals_dict,
};

